--
-- Alignment Helper
--
-- @author  	V!rus(Lukas Reiche)
-- @version 	v1
-- @date  	05/02/13
--

--
-- This mod helps to align vehicles.
--
-- Usage:
-- Ctrl + right Mouse => (de)activate
-- Ctrl + scroll => change size
--
-- If you want to learn from this code, go for it. This is why its commented.
--
-- !! If you want to reuse or modify this code, please ask for permission !!
--	Just send a short mail to somersault[at]ich-habe-fertig.com
--


--
-- Alignment
--
Alignment = {
    distance = 200,
    active = nil,
    width = 0
}


--
-- Alignment.prerequisitesPresent
--
function Alignment.prerequisitesPresent(specializations)
    return true;
end


--
-- Alignment:loadMap
--
function Alignment:loadMap(name)

    -- Only initialize when map is loaded
    if (Alignment.active == nil) then

        Alignment.active = false

    end
end

;

--
-- Alignment:deleteMap
--
function Alignment:deleteMap()
end

--
--  Alignment:mouseEvent
--
function Alignment:mouseEvent(posX, posY, isDown, isUp, button)

    -- (de)activation
    if isDown and button == 3 and Input.isKeyPressed(Input.KEY_lctrl) then
        Alignment.active = not Alignment.active;
    end
    -- changing size (not less then 0!)
    if button == 4 and Input.isKeyPressed(Input.KEY_lctrl) and Alignment.width > 0 then
        Alignment.width = Alignment.width - 0.01;
    end
    if button == 5 and Input.isKeyPressed(Input.KEY_lctrl) then
        Alignment.width = Alignment.width + 0.01;
    end
end

--
-- Alignment:keyEvent
--
function Alignment:keyEvent(unicode, sym, modifier, isDown)
end


--
-- Alignment:update
--
function Alignment:update(dt)

    -- active?
    if Alignment.active then

        -- BaseMission.Alignmentdt = dt;
	
	-- using vehicle?
        if g_currentMission.controlledVehicle ~= nil then

	    -- above ground
            local yOffset = 0.2;
	    -- color
            local r, g, b = 1, 0, 0;
	    -- length of one part of the lines
            local length = 20;

	    -- get direction
            local xDir, yDir, zDir = localDirectionToWorld(g_currentMission.controlledVehicle.rootNode, 0, 0, 1);

	    -- get vehicle cords
            local aX1, aY1, aZ1 = getWorldTranslation(g_currentMission.controlledVehicle.rootNode);
            local bX1, bY1, bZ1 = getWorldTranslation(g_currentMission.controlledVehicle.rootNode);

	    -- make the line start behind vehicle
            aX1 = aX1 - (xDir * 15);
            aY1 = aY1 - (yDir * 15);
            aZ1 = aZ1 - (zDir * 15);

            bX1 = bX1 - (xDir * 15);
            bY1 = bY1 - (yDir * 15);
            bZ1 = bZ1 - (zDir * 15);

	    -- add y Offset
            aY1 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, aX1, aY1, aZ1) + yOffset;
            bY1 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, aX1, aY1, aZ1) + yOffset;

	    -- end points of the parts
            local aX2, aY2, aZ2 = nil, nil, nil;
            local bX2, bY2, bZ2 = nil, nil, nil;

	    -- get orthographic vector to direction
            local x, z = nil, nil;
	    z = - xDir;
	    x = zDir;

	    -- length of this vector
	    local n = math.sqrt((x*x+z*z));

	    -- orthographic gets length of the set width
	    x = x / n * Alignment.width;
	    z = z / n * Alignment.width;

	    -- move one line(starting point of it) to left(right?)
	    aX1 = aX1 + x;
	    aZ1 = aZ1 + z;

	    -- move the other line(starting point of it) in the other direction
	    bX1 = bX1 - x;
	    bZ1 = bZ1 - z;

	    -- draw the single parts of the lines
            for i = 0, math.floor(Alignment.distance / length) + 2 do

		-- get end points of part
                aX2 = aX1 + (xDir * length);
                aY2 = aY1 + (yDir * length);
                aZ2 = aZ1 + (zDir * length);

                bX2 = bX1 + (xDir * length);
                bY2 = bY1 + (yDir * length);
                bZ2 = bZ1 + (zDir * length);

		-- add the Y offset again
                aY2 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, aX2, aY2, aZ2) + yOffset;

                bY2 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, bX2, bY2, bZ2) + yOffset;


		-- draw the lines
                drawDebugLine(aX1, aY1, aZ1, r, g, b, aX2, aY2, aZ2, r, g, b);

                drawDebugLine(bX1, bY1, bZ1, r, g, b, bX2, bY2, bZ2, r, g, b);


		-- end points are the new starting points for the next part
                aX1 = aX2;
                aY1 = aY2;
                aZ1 = aZ2;

                bX1 = bX2;
                bY1 = bY2;
                bZ1 = bZ2;
            end;
        end;
    end;
end

--
-- Alignment:draw
--

function Alignment:draw()
end

--
-- Register Mod Events
--

addModEventListener(Alignment);